"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var resolvePackagesCache = {};
var resolveModulesCache = {};
var KNOWN_RN_PACKAGES = [/expo.*/, /@expo.*/, /react-navigation.*/, /react-native.*/];
exports.default = (function (builder, modulePath) {
    var idx = modulePath.lastIndexOf(path.sep + 'node_modules' + path.sep);
    if (idx >= 0) {
        if (resolveModulesCache[modulePath] === undefined) {
            var pkgPathStart = modulePath[idx + 14] !== '@' ? idx + 14 : modulePath.indexOf(path.sep, idx + 14) + 1;
            var pkgPathEnd = modulePath.indexOf(path.sep, pkgPathStart);
            if (pkgPathEnd < 0) {
                pkgPathEnd = modulePath.length;
            }
            var pkgPath = modulePath.substr(0, pkgPathEnd);
            if (resolvePackagesCache[pkgPath] === undefined) {
                var pkgName_1 = pkgPath.substr(idx + 14);
                var shouldTranspile = KNOWN_RN_PACKAGES.some(function (regex) { return regex.test(pkgName_1); });
                var resolvedPath = pkgPath;
                if (!shouldTranspile) {
                    try {
                        if (fs.lstatSync(pkgPath).isSymbolicLink()) {
                            var realPath = fs.realpathSync(pkgPath);
                            resolvedPath = realPath;
                            shouldTranspile = realPath.indexOf(builder.projectRoot) === 0;
                        }
                    }
                    catch (e) { }
                }
                if (!shouldTranspile) {
                    var entryFileText = void 0;
                    try {
                        entryFileText = fs.readFileSync(builder.require.resolve(pkgName_1), 'utf8');
                    }
                    catch (e) { }
                    shouldTranspile =
                        !entryFileText || entryFileText.indexOf('__esModule') >= 0
                            ? false
                            : /^(export|import)[\s]/m.test(entryFileText);
                }
                resolvePackagesCache[pkgPath] = {
                    realPath: resolvedPath,
                    shouldTranspile: shouldTranspile
                };
            }
            var resolvedPkg = resolvePackagesCache[pkgPath];
            resolveModulesCache[modulePath] = {
                realPath: path.join(resolvedPkg.realPath, modulePath.substr(pkgPathEnd + 1)),
                shouldTranspile: resolvedPkg.shouldTranspile
            };
        }
        return resolveModulesCache[modulePath];
    }
    else {
        return { realPath: modulePath, shouldTranspile: true };
    }
});
//# sourceMappingURL=resolveModule.js.map